searchNodes=[{"doc":"Ecto integration with MongoDB. This document will present a general overview of using MongoDB with Ecto, including common pitfalls and extra functionalities. Check the Ecto documentation for an introduction or examples/simple for a sample application using Ecto and MongoDB. Repositories The first step to use MongoDB with Ecto is to define a repository with Mongo.Ecto as an adapter. First define a module: defmodule Repo do use Ecto.Repo , otp_app : :my_app end Then configure it your application environment, usually in your config/config.exs : config :my_app , Repo , adapter : Mongo.Ecto , database : &quot;ecto_simple&quot; , username : &quot;mongodb&quot; , password : &quot;mongodb&quot; , hostname : &quot;localhost&quot; Each repository in Ecto defines a start_link/0 function that needs to be invoked before using the repository. This function is generally from your supervision tree: def start ( _type , _args ) do import Supervisor.Spec children = [ worker ( Repo , [ ] ) ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end Models With the repository defined, we can define our models: defmodule Weather do use Ecto.Model # see the note below for explanation of that line @primary_key { :id , :binary_id , autogenerate : true } # weather is the MongoDB collection name schema &quot;weather&quot; do field :city , :string field :temp_lo , :integer field :temp_hi , :integer field :prcp , :float , default : 0.0 end end Ecto defaults to using :id type for primary keys, that is translated to :integer for SQL databases, and is not handled by MongoDB. You need to specify the primary key to use the :binary_id type, that the adapter will translate to ObjectID. Remember to place this declaration before the schema call. The name of the primary key is just a convenience, as MongoDB forces us to use _id . Every other name will be recursively changed to _id in all calls to the adapter. We propose to use id or _id as your primary key name to limit eventual confusion, but you are free to use whatever you like. Using the autogenerate: true option will tell the adapter to take care of generating new ObjectIDs. Otherwise you need to do this yourself. Since setting @primary_key for every model can be too repetitive, we recommend you to define your own module that properly configures it: defmodule MyApp.Model do defmacro __using__ ( _ ) do quote do use Ecto.Model @primary_key { :id , :binary_id , autogenerate : true } @foreign_key_type :binary_id # For associations end end end Now, instead of use Ecto.Model , you can use MyApp.Model in your modules. All Ecto types, except :decimal , are supported by Mongo.Ecto . By defining a schema, Ecto automatically defines a struct with the schema fields: iex&gt; weather = % Weather { temp_lo : 30 } iex&gt; weather . temp_lo 30 The schema also allows the model to interact with a repository: iex&gt; weather = % Weather { temp_lo : 0 , temp_hi : 23 } iex&gt; Repo . insert! ( weather ) % Weather { ... } After persisting weather to the database, it will return a new copy of %Weather{} with the primary key (the id ) set. We can use this value to read a struct back from the repository: # Get the struct back iex&gt; weather = Repo . get Weather , &quot;507f191e810c19729de860ea&quot; % Weather { id : &quot;507f191e810c19729de860ea&quot; , ... } # Update it iex&gt; weather = %{ weather | temp_lo : 10 } iex&gt; Repo . update! ( weather ) % Weather { ... } # Delete it iex&gt; Repo . delete! ( weather ) % Weather { ... } Queries Mongo.Ecto also supports writing queries in Elixir to interact with your MongoDB. Let's see an example: import Ecto.Query , only : [ from : 2 ] query = from w in Weather , where : w . prcp &gt; 0 or is_nil ( w . prcp ) , select : w # Returns %Weather{} structs matching the query Repo . all ( query ) Queries are defined and extended with the from macro. The supported keywords in MongoDB are: :where :order_by :offset :limit :select :preload When writing a query, you are inside Ecto's query syntax. In order to access params values or invoke functions, you need to use the ^ operator, which is overloaded by Ecto: def min_prcp ( min ) do from w in Weather , where : w . prcp &gt; ^ min or is_nil ( w . prcp ) end Besides Repo.all/1 , which returns all entries, repositories also provide Repo.one/1 , which returns one entry or nil, and Repo.one!/1 which returns one entry or raises. There is also support for count function in queries that uses MongoDB 's count command. Please note that unlike in SQL databases you can only select a count - there is no support for querying using a count, there is also no support for counting documents and selecting them at the same time. Please note that not all Ecto queries are valid MongoDB queries. The adapter will raise Ecto.QueryError if it encounters one, and will try to be as specific as possible as to what exactly is causing the problem. For things that are not possible to express with Elixir's syntax in queries, you can use keyword fragments: from p in Post , where : fragment ( key : [ &quot;$exists&quot; : true ] ) , select : p To ease of using in more advanced queries, there is Mongo.Ecto.Helpers module you could import into modules dealing with queries. Please see the documentation of the Mongo.Ecto.Helpers module for more information and supported options. Options for reader functions ( Repo.all/2 , Repo.one/2 , etc) Such functions also accept options when invoked which allow you to use parameters specific to MongoDB find function: :slave_ok - the read operation may run on secondary replica set member :partial - partial data from a query against a sharded cluster in which some shards do not respond will be returned in stead of raising error Upserts MongoDB's upsert featureset is not as rich as that found in other databases, and therefore not all features provided for by Ecto are supported. Some upsert operations can be achieved with multiple database calls and are therefore unsafe. A warning is printed to the console in these cases. Since these operations involve multiple DB calls they cannot be considered safe and cannot be recommended. They're intended mostly to make the transition for Ecto developers coming from another database as clean as possible. Other operations are completely impossible without jumping through many database calls and at the time of writing it wasn't deemed worth it to take this much further. In these cases an error is raised and it's up to you to pick another strategy. Of course PRs are always welcome! https://hexdocs.pm/ecto/Ecto.Repo.html#c:insert/2-upserts Commands MongoDB has many administrative commands you can use to manage your database. We support them through the Mongo.Ecto.command/2 function. Mongo.Ecto . command ( MyRepo , createUser : &quot;ecto&quot; , ... ) We also support one higher level command - Mongo.Ecto.truncate/1 that is used to clear the database, i.e. during testing. Mongo.Ecto . truncate ( MyRepo ) You can use it in your setup call for cleaning the database before every test. You can define your own module to use instead of ExUnit.Case , so you don't have to define this each time. defmodule MyApp.Case do use ExUnit.CaseTemplate setup do Mongo.Ecto . truncate ( MyRepo ) :ok end end Please see documentation for those functions for more information. Associations Ecto supports defining associations on schemas: defmodule Post do use Ecto.Model @primary_key { :id , :binary_id , autogenerate : true } @foreign_key_type :binary_id schema &quot;posts&quot; do has_many :comments , Comment end end Keep in mind that Ecto associations are stored in different Mongo collections and multiple queries may be required for retrieving them. While Mongo.Ecto supports almost all association features in Ecto, keep in mind that MongoDB does not support joins as used in SQL - it's not possible to query your associations together with the main model. Some more elaborate association schemas may force Ecto to use joins in some queries, that are not supported by MongoDB as well. One such call is Ecto.Model.assoc/2 function with a has_many :through association. You can find more information about defining associations and each respective association module in Ecto.Schema docs. Embedded models Ecto supports defining relations using embedding models directly inside the parent model, and that fits MongoDB's design perfectly. defmodule Post do #... schema &quot;posts&quot; do embeds_many :comments , Comment end end defmodule Comment do embedded_schema do field :body , :string end end You can find more information about defining embedded models in the Ecto.Schema docs. Indexes and Migrations Although schema migrations make no sense for databases such as MongoDB there is one field where they can be very beneficial - indexes. Because of this Mongodb.Ecto supports Ecto's database migrations. You can generate a migration with: $ mix ecto.gen.migration create_posts This will create a new file inside priv/repo/migrations with the up and down functions. Check Ecto.Migration for more information. Because MongoDB does not support (or need) database schemas majority of the functionality provided by Ecto.Migration is not useful when working with MongoDB. As we've already noted the most useful part is indexing, but there are others - creating capped collections, executing administrative commands, or migrating data, e.g.: defmodule SampleMigration do use Ecto.Migration def up do create table ( :my_table , options : [ capped : true , size : 1024 ] ) create index ( :my_table , [ :value ] ) create unique_index ( :my_table , [ :unique_value ] ) execute touch : &quot;my_table&quot; , data : true , index : true end def down do # ... end end MongoDB adapter does not support create_if_not_exists or drop_if_exists migration functions. MongoDB adapter features The adapter uses mongodb for communicating with the database and a pooling library such as poolboy for managing connections. The adapter has support for: documents with ObjectID as their primary key insert, find, update, remove and count mongo functions management commands with command/2 embedded documents either with :map type, or embedded models partial updates using change_map/2 and change_array/2 from the Mongo.Ecto.Helpers module queries using javascript expression and regexes using respectively javascript/2 and regex/2 functions from Mongo.Ecto.Helpers module. MongoDB adapter options Options passed to the adapter are split into different categories decscribed below. All options should be given via the repository configuration. Compile time options Those options should be set in the config file and require recompilation in order to make an effect. :adapter - The adapter name, in this case, Mongo.Ecto :pool - The connection pool module, defaults to Mongo.Pool.Poolboy :log_level - The level to use when logging queries (default: :debug ) Connection options :hostname - Server hostname (default: localhost ) :port - Server port (default: 27017 ) :username - Username :password - User password :mongo_url - A MongoDB URL :connect_timeout - The timeout for establishing new connections (default: 5000) :w - MongoDB's write convern (default: 1). If set to 0, some of the Ecto's functions may not work properely :j , :fsync , :wtimeout - Other MongoDB's write concern options. Please consult MongoDB's documentation Pool options Mongo.Ecto does not use Ecto pools, instead pools provided by the MongoDB driver are used. The default poolboy adapter accepts following options: :pool_size - The number of connections to keep in the pool (default: 10) :max_overflow - The maximum overflow of connections (default: 0) For other adapters, please see their documentation.","ref":"Mongo.Ecto.html","title":"Mongo.Ecto","type":"module"},{"doc":"Runs a command in the database. Usage Mongo.Ecto . command ( Repo , drop : &quot;collection&quot; ) Options :database - run command against a specific database (default: repo's database) :log - should command queries be logged (default: true) For list of available commands please see: http://docs.mongodb.org/manual/reference/command/","ref":"Mongo.Ecto.html#command/3","title":"Mongo.Ecto.command/3","type":"function"},{"doc":"Creates one or more indexes for the specified collection coll . See https://docs.mongodb.com/manual/reference/method/db.collection.createIndexes/#mongodb-method-db.collection.createIndexes for the syntax of indexes .","ref":"Mongo.Ecto.html#create_indexes/4","title":"Mongo.Ecto.create_indexes/4","type":"function"},{"doc":"Drops the specified indexes in the collection coll . To drop a single index, pass the name of the index. To drop multiple indexes at once pass a list of indexes to index . To drop all indexes except that of _id pass &quot;*&quot; to index . See https://docs.mongodb.com/manual/reference/command/dropIndexes/#dropindexes","ref":"Mongo.Ecto.html#drop_indexes/4","title":"Mongo.Ecto.drop_indexes/4","type":"function"},{"doc":"","ref":"Mongo.Ecto.html#index/4","title":"Mongo.Ecto.index/4","type":"function"},{"doc":"","ref":"Mongo.Ecto.html#list_index_names/3","title":"Mongo.Ecto.list_index_names/3","type":"function"},{"doc":"Lists indexes in the specified repo and collection .","ref":"Mongo.Ecto.html#list_indexes/3","title":"Mongo.Ecto.list_indexes/3","type":"function"},{"doc":"","ref":"Mongo.Ecto.html#row_to_list/2","title":"Mongo.Ecto.row_to_list/2","type":"function"},{"doc":"Drops all the collections in current database. Skips system collections and schema_migrations collection. Especially useful in testing. Returns list of dropped collections.","ref":"Mongo.Ecto.html#truncate/2","title":"Mongo.Ecto.truncate/2","type":"function"},{"doc":"An Ecto type to represent a partial update of a nested document Using in queries change = Mongo.Ecto.Helpers . change_array ( &quot;0.name&quot; , &quot;name&quot; ) MyRepo . update_all Post , set : [ authors : change ]","ref":"Mongo.Ecto.ChangeArray.html","title":"Mongo.Ecto.ChangeArray","type":"module"},{"doc":"Casts to database format","ref":"Mongo.Ecto.ChangeArray.html#cast/1","title":"Mongo.Ecto.ChangeArray.cast/1","type":"function"},{"doc":"Converts to a database format","ref":"Mongo.Ecto.ChangeArray.html#dump/1","title":"Mongo.Ecto.ChangeArray.dump/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"Mongo.Ecto.ChangeArray.html#embed_as/1","title":"Mongo.Ecto.ChangeArray.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"Mongo.Ecto.ChangeArray.html#equal?/2","title":"Mongo.Ecto.ChangeArray.equal?/2","type":"function"},{"doc":"Change is not a value - it can't be loaded","ref":"Mongo.Ecto.ChangeArray.html#load/1","title":"Mongo.Ecto.ChangeArray.load/1","type":"function"},{"doc":"The Ecto primitive type","ref":"Mongo.Ecto.ChangeArray.html#type/0","title":"Mongo.Ecto.ChangeArray.type/0","type":"function"},{"doc":"","ref":"Mongo.Ecto.ChangeArray.html#t:t/0","title":"Mongo.Ecto.ChangeArray.t/0","type":"type"},{"doc":"An Ecto type to represent a partial update of a nested document Using in queries change = Mongo.Ecto.Helpers . change_map ( &quot;name&quot; , &quot;name&quot; ) MyRepo . update_all Post , set : [ author : change ]","ref":"Mongo.Ecto.ChangeMap.html","title":"Mongo.Ecto.ChangeMap","type":"module"},{"doc":"Casts to database format","ref":"Mongo.Ecto.ChangeMap.html#cast/1","title":"Mongo.Ecto.ChangeMap.cast/1","type":"function"},{"doc":"Converts to a database format","ref":"Mongo.Ecto.ChangeMap.html#dump/1","title":"Mongo.Ecto.ChangeMap.dump/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"Mongo.Ecto.ChangeMap.html#embed_as/1","title":"Mongo.Ecto.ChangeMap.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"Mongo.Ecto.ChangeMap.html#equal?/2","title":"Mongo.Ecto.ChangeMap.equal?/2","type":"function"},{"doc":"Change is not a value - it can't be loaded","ref":"Mongo.Ecto.ChangeMap.html#load/1","title":"Mongo.Ecto.ChangeMap.load/1","type":"function"},{"doc":"The Ecto primitive type","ref":"Mongo.Ecto.ChangeMap.html#type/0","title":"Mongo.Ecto.ChangeMap.type/0","type":"function"},{"doc":"","ref":"Mongo.Ecto.ChangeMap.html#t:t/0","title":"Mongo.Ecto.ChangeMap.t/0","type":"type"},{"doc":"Defines helpers to ease working with MongoDB in models or other places where you work with them. You'd probably want to import it.","ref":"Mongo.Ecto.Helpers.html","title":"Mongo.Ecto.Helpers","type":"module"},{"doc":"Allows updating only a fragment of a nested document inside an array Usage in queries MyRepo . update_all ( Post , set : [ comments : change_array ( 0 , &quot;author&quot; , &quot;NewName&quot; ) ] )","ref":"Mongo.Ecto.Helpers.html#change_array/3","title":"Mongo.Ecto.Helpers.change_array/3","type":"function"},{"doc":"Allows updating only a fragment of a nested document Usage in queries MyRepo . update_all ( Post , set : [ meta : change_map ( &quot;author.name&quot; , &quot;NewName&quot; ) ] )","ref":"Mongo.Ecto.Helpers.html#change_map/2","title":"Mongo.Ecto.Helpers.change_map/2","type":"function"},{"doc":"Creates proper regex object that can be passed to the database. Usage in queries from p in Post , where : fragment ( title : ^ regex ( &quot;elixir&quot; , &quot;i&quot; ) ) For supported options please see Mongo.Ecto.Regex module documentation.","ref":"Mongo.Ecto.Helpers.html#regex/2","title":"Mongo.Ecto.Helpers.regex/2","type":"function"},{"doc":"An Ecto type to represent MongoDB's Regex objects Both MongoDB and Elixir use PCRE (Pearl Compatible Regular Expressions), so the syntax is similar for both. Using in queries MongoDB supports two ways to use regexes. One is to use regex object: regex = Mongo.Ecto.Helpers . regex ( &quot;elixir&quot; , &quot;i&quot; ) from p in Post , where : fragment ( title : ^ regex ) The other one is to use the $regex operator: from p in Post , where : fragment ( title : [ &quot;$regex&quot; : &quot;elixir&quot; , &quot;$options&quot; : &quot;i&quot; ] ) For more information about the differences between the two syntaxes please refer to the MongoDB documentation: http://docs.mongodb.org/manual/reference/operator/query/regex/#syntax-restrictions Options i - case insensitive m - causes ^ and $ to mark the beginning and end of each line x - whitespace characters are ignored except when escaped and allow # to delimit comments, requires using $regex operator together with the $option operator s - causes dot to match newlines, requires using $regex operator together with the $option operator","ref":"Mongo.Ecto.Regex.html","title":"Mongo.Ecto.Regex","type":"module"},{"doc":"Casts to database format","ref":"Mongo.Ecto.Regex.html#cast/1","title":"Mongo.Ecto.Regex.cast/1","type":"function"},{"doc":"Converts a Mongo.Ecto.Regex into BSON.Regex","ref":"Mongo.Ecto.Regex.html#dump/1","title":"Mongo.Ecto.Regex.dump/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"Mongo.Ecto.Regex.html#embed_as/1","title":"Mongo.Ecto.Regex.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"Mongo.Ecto.Regex.html#equal?/2","title":"Mongo.Ecto.Regex.equal?/2","type":"function"},{"doc":"Converts a BSON.Regex into Mongo.Ecto.Regex","ref":"Mongo.Ecto.Regex.html#load/1","title":"Mongo.Ecto.Regex.load/1","type":"function"},{"doc":"The Ecto primitive type.","ref":"Mongo.Ecto.Regex.html#type/0","title":"Mongo.Ecto.Regex.type/0","type":"function"},{"doc":"","ref":"Mongo.Ecto.Regex.html#t:t/0","title":"Mongo.Ecto.Regex.t/0","type":"type"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Introduce support for Ecto 3 Introduce GitHub actions, replacing Travis CI Use MongoDB 1.0.0 to add support for Mongodb 4.4. and 5.0 NOTE: This might work with versions of Ecto less than 3.6. Refer to the ecto-3 branch if you need to find a commit that works with a non-officially supported version of Ecto. Possible breaking changes Some upsert operations are only supported with MongoDBs 4.2 or newer.","ref":"changelog.html#1-0-0","title":"Changelog - 1.0.0","type":"extras"},{"doc":"Support MongoDB version 3.2","ref":"changelog.html#v0-1-4-2016-03-03","title":"Changelog - v0.1.4 (2016-03-03)","type":"extras"},{"doc":"This version is limited to Ecto 1.0 because of known issues with 1.1 Additions: Implement count(field, :distinct) Bug fixes: Handle models without autogenerated primary key on update and delete Implement Ecto.Adapter.stop/2 callback Move encoding to adapter load and dump callbacks","ref":"changelog.html#v0-1-3-2016-01-15","title":"Changelog - v0.1.3 (2016-01-15)","type":"extras"},{"doc":"Breaking changes: Raise on limit and offset in update_all and delete_all queries, it's not supported by MongoDB, we were failing siletnly before Bug fixes: Allow interpolation in limit and offset","ref":"changelog.html#v0-1-2-2015-10-18","title":"Changelog - v0.1.2 (2015-10-18)","type":"extras"},{"doc":"Bug fixes: Fix logging issues on find queries","ref":"changelog.html#v0-1-1-2015-08-30","title":"Changelog - v0.1.1 (2015-08-30)","type":"extras"},{"doc":"First release","ref":"changelog.html#v0-1-0-2015-08-25","title":"Changelog - v0.1.0 (2015-08-25)","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS","ref":"license.html","title":"License","type":"extras"},{"doc":"Mongo.Ecto is a MongoDB adapter for Ecto. For detailed information read the documentation for the Mongo.Ecto module, or check out examples below.","ref":"readme.html","title":"Overview","type":"extras"},{"doc":"# In your config/config.exs file config :my_app , Repo , adapter : Mongo.Ecto , database : &quot;ecto_simple&quot; , username : &quot;mongodb&quot; , password : &quot;mongosb&quot; , hostname : &quot;localhost&quot; # In your application code defmodule Repo do use Ecto.Repo , otp_app : :my_app end defmodule Weather do use Ecto.Model @primary_key { :id , :binary_id , autogenerate : true } schema &quot;weather&quot; do field :city # Defaults to type :string field :temp_lo , :integer field :temp_hi , :integer field :prcp , :float , default : 0.0 end end defmodule Simple do import Ecto.Query def sample_query do query = from w in Weather , where : w . prcp &gt; 0 or is_nil ( w . prcp ) , select : w Repo . all ( query ) end end","ref":"readme.html#example","title":"Overview - Example","type":"extras"},{"doc":"Add :mongodb_ecto as a dependency in your mix.exs file. def deps do [ { :mongodb_ecto , &quot;~&gt; 1.0.0&quot; } ] end You should also update your applications to include both projects: def application do [ applications : [ :logger , :mongodb_ecto , :ecto ] ] end To use the adapter in your repo: defmodule MyApp.Repo do use Ecto.Repo , otp_app : :my_app , adapter : Mongo.Ecto end For additional information on usage please see the documentation for Ecto .","ref":"readme.html#usage","title":"Overview - Usage","type":"extras"},{"doc":"BSON Ecto double :float string :string object :map array {:array, subtype} binary data :binary binary data (uuid) Ecto.UUID object id :binary_id boolean :boolean date Ecto.DateTime regular expression Mongo.Ecto.Regex JavaScript Mongo.Ecto.JavaScript symbol (see below) 32-bit integer :integer timestamp BSON.Timestamp 64-bit integer :integer Symbols are deprecated by the BSON specification . They will be converted to simple strings on reads. There is no possibility of persisting them to the database. Additionally special values are translated as follows: BSON Ecto null nil min key :BSON_min max key :BSON_max","ref":"readme.html#data-type-mapping","title":"Overview - Data Type Mapping","type":"extras"},{"doc":"The adapter and the driver are tested against most recent versions from 2 branches: 4.4 and 5.0","ref":"readme.html#supported-mongo-versions","title":"Overview - Supported Mongo versions","type":"extras"},{"doc":"To contribute you need to compile Mongo.Ecto from source and test it: $ git clone https://github.com/ankhers/mongodb_ecto.git $ cd mongodb_ecto $ mix test","ref":"readme.html#contributing","title":"Overview - Contributing","type":"extras"},{"doc":"Copyright 2015 Michał Muskała Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at https://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#copyright-and-license","title":"Overview - Copyright and License","type":"extras"}]